!  Propagating chemical reactions forward in time. Example 1.
!
!  Consider a series of elementary reactions.  To be concrete, we 
!       consider a very simple one
!
!       A <--> B    (forward rate: kab; bckward: kba)
!
!       B <--> 2C   (forward: kbc; backward: kcb)
!
!       C --> D     (forward: kcd; no backward)
!
!   Since (AND ONLY SINCE) these are elemntary reactions, we
!       can write immiedieately the rate laws:
!         
!       d[A]/dt = -kab [A]+kba [B}
!       d[B]/dt = +kab [A]-kba [B]  -kbc [B}+  kcb [C]**2
!       d[C]/dt =                 -2*kbc [B}+2 kcb [C]**2 -kcd [C]
!       d[D]/dt =                                         +kcd [C]
!
!   How to propagate forward these reactions numerically? 
!
!   The absolute simplest way is from Euler'rs law, e.g.,
!   a simple Talor's expansion: 
!
!   d[A]/dt ~ ([A(t+dt)]-[A(t)])/dt -->
!
!   [A(t+dt)] ~ [A(t)]+ dt*    d[A]/dt   
!
!   and similarly for [B].  Much better approaches would be 
!   shown later on.
!
!   So let's simulate.  
!       
!   
program chemical_reactions_simple
  implicit none

  !
  !   First let's write parameters, etc.  Here we'll use parameters
  !   close to 1.  (later examples would play with the parameters)
  
  ! Note that we use the "parameter" statement.  It enusres that the program
  ! will never change the values of the parameters -- it is very handy
  ! in subroutines if you want to fix a parameter and ensure it does 
  ! not change
  
  real*8, parameter :: kab = 1, kba = 3, kbc = 4.2, kcb = 7.3, kcd = 0.4
    
  real*8:: a, b, c, d  !densities--safer to use 2-character varaible names
  real*8:: da_dt, db_dt, dc_dt, dd_dt

  real*8:: dt, tmax, time
  
  integer nt, it
  
  !
  !   Now decide on time step and total time.  We'll need to verify that
  !   we converge the results w.r.t. the time-step
  
  
  dt = 0.0001;
  tmax = 60;       ! total time
  nt = tmax/dt;  !  number of time steps
  
  a = 5        ! initial density
  b = 0
  c = 0
  d = 0

  !
  !   Now let's start running; we'll propagate one step at a time.
  !
  time = 0;
  do it=1,nt
     !
     !  First calculate da/dt, etc.
     !
     time = time + dt;
     
     da_dt = -kab*a+kba*b;
     db_dt = +kab*a-kba*b -kbc*b +    kcb*c**2;
     dc_dt =               2*kbc*b -2*kcb*c**2 -kcd*c;
     dd_dt =                                   +kcd*c;
     
     !
     ! Now use Taylor's expansion to propagate
     !  a(t+dt)=a(t)+dt*da_dt, i.e., a_new = a_old + dt*da_dt
     
     a = a + dt*da_dt;
     b = b + dt*db_dt;
     c = c + dt*dc_dt;
     d = d + dt*dd_dt;
     
  end do
  
  !
  !   Now let's display the results (for better plotting see next
  !   few examples)  (See class for format explanation)
  write(6,*) &
  ' dt          time         a           b              c              d '
  write(6,'(1x,6e18.10)')dt,time,a,b,c,d
   
  !
  !  HOMEWORK: change dt, and see what results do you get- how is the
  !   convergence?  
  !

end
  
