!
! Matrix programs.
! Here are the two matrix programs that you'll need:
!  mat_inv   and mat_diag     as well as plot_mat
!
! To use these routines, place the statement      
!                   use mat_module
! in your program, immideately after the 'program' or 'subroutine' statement
! (before the implicit none).
! 
! Calling:
!
!
!  call mat_inv(A,A_inv)
! 
!        where A (input) is a general real*8 matrix, and so is A_inv (output).
!     OR:
!       A and A_inv can be both complex*16 matrices.
!
!       A and A_inv should be of the same size (N*N, where N is the # of
!       rows of A)
!
!
!
! call mat_diag(A,A_vc, A_evl)
!
!        where A (input) is a SYMMETRIC real*8 matrix
!              A_vc (ouput) is a (non-symmetric, but orthogonal) real*8 matrix
!                     (i.e., the eigenvector matrix)
!         and 
!              A_evl  (output) is a list of eigenvalues .
!
!        Note that A_evl is NOT the diagonal eigevnlue matrix, D,
!        defined as 
!                   D=0; do i=1,N; D(i,i)=A_evl(i); enddo
!
!        The eigenvalue equation is matmul(A,A_vc) = matmul(A_vc, D)
!
!        That equation is checked.
!
!        Finally: this (diagonalization) routine is not yet set up for complex 
!          (gfeneral or hermitian) matrices -- I'll do it later.
!
!
module mat_module
  interface mat_inv
     module procedure mat_inv_r8
     module procedure mat_inv_c16
  end interface
  
  interface mat_diag
     module procedure mat_diag_r8_symm
  end interface

  interface plot_mat
     module procedure plot_mat_r8
     module procedure plot_mat_c16
  end interface
contains
  subroutine mat_inv_r8(A, A_inv)
    implicit none
    real*8 A(:,:), A_inv(:,:)
    real*8, allocatable ::  Unity(:,:)
    integer N, M, st, i
    integer, parameter :: flag_check = 1   
    real*8  diff

    N = Ubound(A, 1)- Lbound(A,1) + 1
    M = Ubound(A, 2)- Lbound(A,2) + 1
    
    if(M/=N.or.size(A_inv,1)/=N.or.size(A_inv,2)/=N) then
       write(6,*)' problem with sizes in mat_diag_r8_symm '
       write(6,*)' size(A) ',size(A)
       write(6,*)' size(A_inv) ',size(A_inv)
       stop
    endif

    call r_inv_mtrx(A, A_inv, N)
    if(flag_check /= 1) return

!
!   Pre checking: prepare unity
!
    allocate(unity(N,N), stat=st)
    if(st/=0) then
       write(6,*)' problem allocating diag_evl, unity in mat_diag '
       stop
    endif

    Unity    = 0.d0
    do i=1, N
       Unity(   i,i) = 1.d0
    enddo
!
!   Now check
!
    diff = sum(abs(matmul(A, A_inv)-unity))

    write(6,*)' deviation from A*Ainv = 1 ',diff
    if(diff>1.d-4) then
       write(6,*)' deviation too big -- stopping '
       stop
    endif
    write(6,*)
!
!    space clearing 
!    

    deallocate(Unity)
  end subroutine mat_inv_r8

  subroutine mat_inv_c16(A, A_inv)
    implicit none
    complex*16 A(:,:), A_inv(:,:)
    real*8, allocatable ::  Unity(:,:)
    integer N, M, st, i
    integer, parameter :: flag_check = 1   
    real*8  diff

    N = Ubound(A, 1)- Lbound(A,1) + 1
    M = Ubound(A, 2)- Lbound(A,2) + 1
    
    if(M/=N.or.size(A_inv,1)/=N.or.size(A_inv,2)/=N) then
       write(6,*)' problem with sizes in mat_diag_r8_symm '
       write(6,*)' size(A) ',size(A)
       write(6,*)' size(A_inv) ',size(A_inv)
       stop
    endif

    call c_inv_mtrx(A, A_inv, N)
    if(flag_check /= 1) return

!
!   Pre checking: prepare unity
!
    allocate(unity(N,N), stat=st)
    if(st/=0) then
       write(6,*)' problem allocating diag_evl, unity in mat_diag '
       stop
    endif

    Unity    = 0.d0
    do i=1, N
       Unity(   i,i) = 1.d0
    enddo
!
!   Now check
!
    diff = sum(abs(matmul(A, A_inv)-unity))

    write(6,*)' deviation from A*Ainv = 1 ',diff
    if(diff>1.d-4) then
       write(6,*)' deviation too big -- stopping '
       stop
    endif
    write(6,*)
!
!    space clearing 
!    

    deallocate(Unity)
  end subroutine mat_inv_c16

  subroutine mat_diag_r8_symm(A, A_vc, A_evl)
    implicit none
    real*8 A(:,:), A_vc(:,:), A_evl(:)
    real*8, allocatable :: Diag_evl(:,:), unity(:,:)
    real*8  diff
    integer N, M,st,i
    integer, parameter :: flag_check = 1   

!
!   sizes 
!
    N = Ubound(A, 1)- Lbound(A,1) + 1
    M = Ubound(A, 2)- Lbound(A,2) + 1

    if(M/=N.or.size(A_vc,1)/=N.or.size(A_vc,2)/=N.or.size(A_evl)/=N) then
       write(6,*)' problem with sizes in mat_diag_r8_symm '
       write(6,*)' size(A) ',size(A)
       write(6,*)' size(A_vc) ',size(A_vc)
       write(6,*)' size(A_evl) ',size(A_evl)
       stop
    endif
    
!
!   call a ready made program
!
    call r_diag_normlz(A, A_vc, A_evl, N)

    if(flag_check /= 1) return
!
!   pre-checking: prepare Diag_evl, Unity
!

    allocate(Diag_evl(N,N), unity(N,N), stat=st)
    if(st/=0) then
       write(6,*)' problem allocating diag_evl, unity in mat_diag '
       stop
    endif

    Unity    = 0.d0
    Diag_evl = 0.d0
    do i=1, N
       Diag_evl(i,i) = A_evl(i)
       Unity(   i,i) = 1.d0
    enddo
!
!   first checking the A_vc and A_evl are indeed e.vectors and e.values.
!

    diff = sum(abs(matmul(A, A_vc)-matmul(A_vc, Diag_evl)))

    write(6,*)' deviation from A*Avc=Avc*Diag ',diff
    if(diff>1.d-4) then
       write(6,*)' deviation too big -- stopping '
       stop
    endif
    write(6,*)
!
!   now checking that A_vc is orthogonal (A_vc*A_vc_tranpose=I)
!

    diff = sum(abs(matmul(A_vc, transpose(A_vc))-unity))

    write(6,*)' deviation from A_vc being orthogonal, i.e., '
    write(6,*)' A_vc*A_vc_transpoe=I   is ',diff
        if(diff>1.d-4) then
       write(6,*)' deviation too big -- stopping '
       stop
    endif
    write(6,*)
!
!    space clearing 
!    

    deallocate(Diag_evl, Unity)
  end subroutine mat_diag_r8_symm

  subroutine plot_mat_c16(A, iplot)
    implicit none
    complex*16 A(:,:)
    integer N,M, i, iplot
    N = size(A,1)
    M = size(A,2)
    if(N<1.or.M<1.or.N*M>1000) then
       write(6,*)' not set to plot matrix with dimensions ',N,M
       stop
    endif
    
    do i=lbound(A,1), ubound(A,1)
       write(iplot,77)A(:,i)
       if(i>3)write(iplot,*)
77     format(' ',3(f11.6,'+i*',f11.6,' '))
    enddo
  end subroutine plot_mat_c16

  subroutine plot_mat_r8(A, iplot)
    implicit none
    real*8 A(:,:)
    integer N,M, i, iplot
    N = size(A,1)
    M = size(A,2)
    if(N<1.or.M<1.or.N*M>1000) then
       write(6,*)' not set to plot matrix with dimensions ',N,M
       stop
    endif
    
    do i=lbound(A,1), ubound(A,1)
       write(iplot,77)A(:,i)
       if(i>6)write(iplot,*)
77     format(' ',6f12.6)
    enddo
  end subroutine plot_mat_r8
end module

       








